#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_TFTLCD.h> // Hardware-specific library


#define LCD_CS A3 // Chip Select goes to Analog 3
#define LCD_CD A2 // Command/Data goes to Analog 2
#define LCD_WR A1 // LCD Write goes to Analog 1
#define LCD_RD A0 // LCD Read goes to Analog 0

#define LCD_RESET A4 // Can alternately just connect to Arduino's reset pin

#define  BLACK   0x0000
#define BLUE    0x001F
#define RED     0xF800
#define GREEN   0x07E0
#define CYAN    0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define WHITE   0xFFFF

Adafruit_TFTLCD tft(LCD_CS, LCD_CD, LCD_WR, LCD_RD, LCD_RESET);
unsigned char mycount = 0;
int sec = 59;
int minute;
int klem;
String mystring = "";
int thenum;

void setup(void)
{ 
  Serial.begin(9600);
  //Serial.println(F("TFT LCD test"));
  #ifdef USE_ADAFRUIT_SHIELD_PINOUT
  Serial.println(F("Using Adafruit 2.8\" TFT Arduino Shield Pinout"));
 #else
  Serial.println(F("Using Adafruit 2.8\" TFT Breakout Board Pinout"));
 #endif

  Serial.print("TFT size is "); Serial.print(tft.width()); Serial.print("x"); Serial.println(tft.height());

  tft.reset();
  uint16_t identifier = tft.readID();
  
  if(identifier == 0x9325) {
    Serial.println(F("Found ILI9325 LCD driver"));
  } else if(identifier == 0x9327) {
    Serial.println(F("Found ILI9327 LCD driver"));
  } else if(identifier == 0x9328) {
    Serial.println(F("Found ILI9328 LCD driver"));
  } else if(identifier == 0x7575) {
    Serial.println(F("Found HX8347G LCD driver"));
  } else if(identifier == 0x9341) {
    Serial.println(F("Found ILI9341 LCD driver"));
  } else if(identifier == 0x8357) {
    Serial.println(F("Found HX8357D LCD driver"));
  } else if(identifier == 0x0154) {
    Serial.println(F("Found S6D0154 LCD driver"));
  } else {
    Serial.print(F("Unknown LCD driver chip: "));
    Serial.println(identifier, HEX);
    Serial.println(F("If using the Adafruit 2.8\" TFT Arduino shield, the line:"));
    Serial.println(F("  #define USE_ADAFRUIT_SHIELD_PINOUT"));
    Serial.println(F("should appear in the library header (Adafruit_TFT.h)."));
    Serial.println(F("If using the breakout board, it should NOT be #defined!"));
    Serial.println(F("Also if using the breakout, double-check that all wiring"));
    Serial.println(F("matches the tutorial."));
    return;
  }
  
  tft.begin(identifier);
  Serial.println(F("Benchmark                Time (microseconds)"));
  Serial.print(F("Circles (filled)         "));
  Serial.println(testFilledCircles(10, MAGENTA));

  Serial.print(F("Circles (outline)        "));
  Serial.println(testCircles(10, WHITE));
  delay(1000);
  tft.setRotation(1);
  tft.fillScreen(BLACK);
  tft.setCursor(20, 20);
  tft.setTextColor(BLUE);  tft.setTextSize(3);
  tft.println("   Welcome to PGT");
  tft.println("  ");
  tft.setTextColor(GREEN);  tft.setTextSize(2);
  tft.println("            LOADING...");
  
  delay(10000);
  Serial.println(F("Done!"));
    
}

void loop()
{
  mygo:
  Serial.println("standing by");
  while(Serial.available())
  {
      delay(10);
      char mychar = Serial.read();
      mycount = mychar;
      if (mychar == '\0')
         break;
      mystring += mychar;
  }
   if(mystring.length() >= 25)
   {   
       Serial.println("long text displaying");
       tft.fillScreen(BLACK);
       Serial.print(F("Circles (filled)         "));
      Serial.println(testFilledCircles(10, MAGENTA));

      Serial.print(F("Circles (outline)        "));
      Serial.println(testCircles(10, WHITE));
      delay(1000);
      
        tft.fillScreen(BLACK);
        tft.setRotation(1);
        tft.setCursor(10, 10);
        tft.setTextColor(BLUE); 
        tft.setTextSize(2);
        tft.println(mystring);
        delay(10000);
        mystring = "";
        goto mygo;
    }
    if (mystring == "clearall")
    { Serial.println("cleared");
      tft.setRotation(1);
      tft.fillScreen(BLACK);
      tft.setCursor(20, 20);
      tft.setTextColor(BLUE);  tft.setTextSize(3);
      tft.println("   Welcome to PGT");
      tft.println("  ");
      tft.setTextColor(GREEN);  tft.setTextSize(2);
      tft.println("            LOADING...");
      delay(10000);
      mystring = "";
      goto mygo;
    }
    if(mystring.length() > 2 && mystring.length() <= 16)
     {   Serial.println("Displating");
         tft.fillScreen(BLACK);
         Serial.print(F("Circles (filled)         "));
         Serial.println(testFilledCircles(10, MAGENTA));

         Serial.print(F("Circles (outline)        "));
         Serial.println(testCircles(10, WHITE));
         delay(1000);
         
         tft.fillScreen(BLACK);
         tft.setRotation(1);
         tft.setCursor(20,20);
         tft.setTextColor(BLUE);
         tft.setTextSize(4);
         tft.println(mystring);
         delay(10000);
         mystring = "";
         goto mygo;
     }
    
    else if(mystring.length() == 1)
    { Serial.println("counting..");
      int myrun = 0;  
      klem = (int)mycount - 48;
      if ((klem == 3) || (klem == 4))
      { 
        thenum = klem * 10;
        for(minute = thenum; minute >= 00; minute--)
          {
            if(myrun == 1)
                break;
            for(sec = 59;sec >= 0; sec--)
            {
              tft.fillScreen(BLACK);
              tft.setRotation(1);
              tft.setCursor(20,20);
              tft.setTextColor(GREEN);
              tft.setTextSize(4);
              tft.println(" Time Remaining");
              tft.print("      ");
              tft.print(minute);
              int val1,val2;
              tft.print(":");
              val1 = sec/10;
              tft.print(val1);
              val2 = sec%10;
              tft.print(val2);
              if(Serial.available()){
               myrun = 1;
               break;
              }
              delay(1000);
            }
          }
          tft.fillScreen(BLACK);
          tft.setRotation(1);
          tft.setCursor(20,20);
          tft.setTextColor(RED);
          tft.setTextSize(5);
          tft.println(" Time up!");
          delay(5000);
          mystring= "";
          goto mygo;
      }
      else if(klem > 4 && klem <=9)
      {
          for(minute = klem; minute >= 00; minute--)
          {
            if(myrun == 1)
                break;
            for(sec = 59;sec >= 0; sec--)
            {
              tft.fillScreen(BLACK);
              tft.setRotation(1);
              tft.setCursor(20,20);
              tft.setTextColor(GREEN);
              tft.setTextSize(4);
              tft.println(" Time Remaining");
              tft.print("      ");
              tft.print(minute);
              int val1,val2;
              tft.print(":");
              val1 = sec/10;
              tft.print(val1);
              val2 = sec%10;
              tft.print(val2);
              if(Serial.available()){
               myrun = 1;
               break;
              }
              delay(1000);
            }
          }
          tft.fillScreen(BLACK);
          tft.setRotation(1);
          tft.setCursor(20,20);
          tft.setTextColor(RED);
          tft.setTextSize(5);
          tft.println(" Time up!");
          delay(5000);
          mystring= "";
          goto mygo;
      }
     else if ((klem >= 0) && (klem <=2))
     {
       for(minute = klem; minute >= 00; minute--)
          {
            if(myrun == 1)
                break;
            for(sec = 59;sec >= 0; sec--)
            {
              tft.fillScreen(BLACK);
              tft.setRotation(1);
              tft.setCursor(20,20);
              tft.setTextColor(GREEN);
              tft.setTextSize(4);
              tft.println(" Time Remaining");
              tft.print("      ");
              tft.print(minute);
              int val1,val2;
              tft.print(":");
              val1 = sec/10;
              tft.print(val1);
              val2 = sec%10;
              tft.print(val2);
              if(Serial.available()){
               myrun = 1;
               break;
              }
              delay(1000);
            }
          }
          tft.fillScreen(BLACK);
          tft.setRotation(1);
          tft.setCursor(20,20);
          tft.setTextColor(RED);
          tft.setTextSize(5);
          tft.println(" Time up!");
          delay(5000);
          mystring= "";
          goto mygo;
     }
      }
   
}

unsigned long testFilledCircles(uint8_t radius, uint16_t color) {
  unsigned long start;
  int x, y, w = tft.width(), h = tft.height(), r2 = radius * 2;

  tft.fillScreen(BLACK);
  start = micros();
  for(x=radius; x<w; x+=r2) {
    for(y=radius; y<h; y+=r2) {
      tft.fillCircle(x, y, radius, color);
    }
  }

  return micros() - start;
}

unsigned long testCircles(uint8_t radius, uint16_t color) {
  unsigned long start;
  int           x, y, r2 = radius * 2,
                w = tft.width()  + radius,
                h = tft.height() + radius;

  // Screen is not cleared for this one -- this is
  // intentional and does not affect the reported time.
  start = micros();
  for(x=0; x<w; x+=r2) {
    for(y=0; y<h; y+=r2) {
      tft.drawCircle(x, y, radius, color);
    }
  }

  return micros() - start;
}
